#!/usr/local/bin/python
# Rob Dennis - Lurchington Q. Delacroix
# Keasby - TV renamer
# Project Homepage: http://code.google.com/p/keasby/

import logging

class xmlDomInteractor():
    def __init__(self,xmldom,keyList=[],linkedInstances=[]):
        self.xml=xmldom
        self.foundKeys=[]
        for key in keyList:
            self.setDictKey(key)
        
        self.max={}
        self.links=linkedInstances
            
    def set_max_value(self,tagname,manual_max_value=None):
        """aids in handling requests for erroneous data, eg: season 451"""

        if not self.links:
            self.max[tagname] = None
        
        elif manual_max_value is not None:
            self.max[tagname] = manual_max_value
            
        else:
            self.max[tagname] = self.links[0][tagname]
            
            for linked_tvdb_instance in self.links:
                
                tagname_value = linked_tvdb_instance[tagname]
                
                if tagname_value > self.max[tagname]:
                    self.max[tagname] = tagname_value
                    
    def get_max_value(self,tagname):
        """returns maximum value for a link, as set in set_max_value"""
        return self.max.get(tagname)
            

    def getChildData(self,tagName,index=-1):
        try:
            foundChildData=self.xml.getElementsByTagName(tagName)[index].firstChild.data
            self.foundKeys.append(tagName)
            return foundChildData
        except IndexError:
            return None
        except AttributeError:
            return None
        except:
            # print "unknown error in getChildData"
            # print "tagname=%s"%(tagName)
            # print "----------"
            # print self.xml.toxml()
            # for element in self.xml.getElementsByTagName(tagName):
                # print element
                # print element.toxml()
            raise
            
    def setDictKey(self,tagName):
        value=self.getChildData(tagName)
        if value and value.isdigit():
            # print "reshaping %s %s to int"%(tagName,value)
            value=int(value)
        self.__dict__[tagName]=value

    def __getitem__(self,key):
        return self.__dict__.get(key)
    
    def __setitem__(self,key,value):
        self.__dict__[key]=value
        
    def __str__(self):
        """meaningful string representation"""
        state = ["%s=%r" % (key,self.__getitem__(key))
                 for key in self.foundKeys]

        return '\n'.join(state)
        
        
class tvDbSeries(xmlDomInteractor):
    def __init__(self,xmlDom,keyList=["seriesid","SeriesName","FirstAired"],linkedInstances=[]):
        self.order={}
        xmlDomInteractor.__init__(self,xmlDom,keyList,linkedInstances)
        self.set_max_value("SeasonNumber")
    
    def orderBy(self,category,targetValue=None):
        # print "attempting to order by %s"%category
        if targetValue is None:
            for episode in self.links:
                if episode[category]:
                    self.addNode(category,episode[category],episode)
        else:
            # print "with target value %s of type %s"%(targetValue,type(targetValue))
            for episode in self.links:
                # print "episode[%s]='%s' of %s"%(category,episode[category],type(episode[category]))
                if episode[category]==targetValue:
                    self.addNode(category,episode[category],episode)

                    
    def addNode(self,category,value,link):
        if self.order.get(category) is None:
            self.order[category]={}
            
        if self.order[category].get(value) is None:
            self.order[category][value] = []
            
        self.order[category][value].append(link)
        
        
    def __str__(self):
        return "\n".join( [link for link in self.links] )
        
class tvDbEpisode(xmlDomInteractor):
    def __init__(self,xmlDom,keyList=["EpisodeName","id","FirstAired","seasonid","FirstAired","EpisodeNumber","SeasonNumber","ProductionCode"]):
        xmlDomInteractor.__init__(self,xmlDom,keyList)
        
class tvDbMirror(xmlDomInteractor):
    def __init__(self,xmlDom,keyList=["id","mirrorpath","typemask"]):
        xmlDomInteractor.__init__(self,xmlDom,keyList)
        