#!/usr/local/bin/python
# Rob Dennis - Lurchington Q. Delacroix
# Keasby - TV renamer
# Project Homepage: http://code.google.com/p/keasby/

#Base Class for a source used for parsing
import os,glob,sys
from keasby_config import KeasbyConfigParser
import urllib2
import traceback

class ParserSrc():
    """Base class for parsing sources expecting: parserSrc(string location=None)"""
    def __init__(self,*args,**kwargs):
        pass

    def initCache(self,location=None):
        """initializes the cache, subclasses should handle exceptions"""
        pass
    
    def dumpSrc(self,location):
        """intended to return contents of location given the knowledge on how to accomplish this.
        Should pass through all exceptions"""
        return None
    
    def getCache(self):
        """returns cache contents"""
        return self._cache
        
    def getValue(key):
        """convenience function for getting dict entries"""
        return self.__dict__.get(key)

        
class MultiParserSrc(ParserSrc):
    """parser instance, based on a cache dictionary"""
    def __init__(self):
        self.initCache()
    
    def initCache(self):
        self._cacheDict={}

    def flushCache(self,location,text,mode="wb"):
        """intended to write cache to file at give location"""
        tmpFile=open(location,mode)
        tmpFile.write(text)
        tmpFile.close()    
        
    def getCache(self,key=None):
        if key:
            return self._cacheDict.get(key)
        else:
            return self._cacheDict
    
    def getCacheDict(self):
        return self._cacheDict
        
    
    

class SingleParserSrc(ParserSrc):
    """parser instance, based on a cache string"""
    def __init__(self,location):
        self.setLocation(location)
        self.initCache(location)
        
    def setLocation(self,location):
        self.location=location
        
    def initCache(self,location=None):
        """intended to be overridden to handle errors"""
        pass

    def flushCache(self,location,mode="wb"):
        """intended to write cache to file at give location"""
        tmpFile=open(location,mode)
        tmpFile.write(self.__cache)
        tmpFile.close()

###---------File-----------    
class FileBase():
    def dumpSrc(self,location):
        """passes through exceptions at the cost of relying on garbage collection to reclaim
        files"""
        return open(location,"rb").read()    

class SingleFileSrc(FileBase,SingleParserSrc):
    def initCache(self,location=None):
        try:
            self._cache=self.dumpSrc(location)
            return self._cache
        except IOError:
            print "failed to init cache: %s -> %s"%(sys.exc_info()[0].__name__,sys.exc_info()[1])
            self._cache=None
            return sys.exc_info()
        except TypeError:
            print "failed to init cache: %s -> %s"%(sys.exc_info()[0].__name__,sys.exc_info()[1])
            self._cache=None
            return sys.exc_info()
        except:
            print "failed to init cache: %s -> %s"%(sys.exc_info()[0].__name__,sys.exc_info()[1])
            self._cache=None


        
class MultiFileSrc(FileBase,MultiParserSrc):
    pass


        
###-------URL----------
class UrlBase():
    def dumpSrc(self,location=None):
        """passes through exceptions"""
        return urllib2.urlopen(location).read()
        
class SingleUrlSrc(UrlBase,SingleParserSrc):
    def initCache(self,location=None):
        try:
            self._cache=self.dumpSrc(location)
            return self._cache
        except ValueError:
            self._cache=None
            print r"consider adding http://"
            print "failed to init cache: %s -> %s"%(sys.exc_info()[0].__name__,sys.exc_info()[1])
        except:
            self._cache=None
            print "failed to init cache: %s -> %s"%(sys.exc_info()[0].__name__,sys.exc_info()[1])

class MultiUrlSrc(UrlBase,MultiParserSrc):
    pass
        
###-----tvSource-------

class tvUrlSrc(MultiUrlSrc):
    def __init__(self,configFile):
        tmpConfig=KeasbyConfigParser()
        tmpConfig.read(configFile)
        self.configDict=tmpConfig.getDict(True)
        MultiUrlSrc.__init__(self)

###-----tvApiSource-------

class tvApiSrc():
    def __init__(self,handler):
        self.handler=handler
    
    def getTitle(self,dict)
        











##def main():
##    command_list    = ([
###                         (SingleFileSrc,None),
###                         (SingleFileSrc,sys.argv[0]),
###                         (SingleFileSrc,"DNE_file.imaginary"),
###                         (SingleUrlSrc,"http://www.epguides.com"),
###                         (SingleUrlSrc,"www.epguides.com"),
###                         (SingleUrlSrc,"http://www.epguides.com/DNE_IMAGINARY"),
##                        (tvUrlSrc,os.path.join("resources","tvComWebHTML.source"))
##                        ])
##                        
##    for command in command_list:
##        try:
##            print "len(%s(%s).GetCache())"%(str(command[0])[len("__main__."):],command[1])
##            print "-"
##            cache=command[0](command[1]).getCache()
##        except:
##            print "failed: %s -> %s"%(sys.exc_info()[0].__name__,sys.exc_info()[1])
##            print traceback.print_tb(sys.exc_info()[2])
##        if type({}) == type(cache):
##            print "successfully filled cache with dict size %s"%len(cache.keys())
##        elif cache:
##            print "successfully filled cache with length %s"%len(cache)
##        else:
##            print "returns None"
##        print "------"
##
##main()