#!/usr/local/bin/python
# Rob Dennis - Lurchington Q. Delacroix
# Keasby - TV renamer
# Project Homepage: http://code.google.com/p/keasby/

#---Python Core
import shutil
import os
import re
import logging
from string import Template
#---Keasby
#from keasby_source import KeasbyInfoRepo
from keasby_options import cl_opt,cl_arg
import media_interpreter

def scrub_illegal(string):
    if string:
        return re.sub(r"[\:\!\<\>\+\=\;\[\]\?\*]","",string)
    else:
        return string

class TvClass():
    def __init__(self,information_repository,**kwargs):
        for key,value in kwargs.items():
            self.set_value(key,value)
        self.init_sources(information_repository)
#---Initialization
    def init_sources(self,information_repository):
        self.repo=information_repository
        
#---Utility
    def get_all(self):
        return self.__dict__
        
    def get_string(self,key,padding=2):
        value=self.getValue(key)
        if str(value).isdigit():
            return str(value).zfill(padding)
        elif value:
            return str(value)
        else:
            return None
        

        
    def set_value(self,key,value):
        if str(value).isdigit():
            value=int(value)
        elif not key.endswith("path"):
            #reshaping for puntuation, but not for path names
            value=re.sub(r"([^'\w]|_)+"," ",value)
            value=str(value).strip().title()
          
        self.__dict__[key]=value
        
    def __getitem__(self,key):
        return self.__dict__.get(key)
    
    def __setitem__(self,key,value):
        self.set_value(key,value)
    
    def __str__(self):
        state = ["%s=%s"%(key,value) for key,value in self.__dict__.items() ]
        return "\n".join(state)

class TvEpisode(TvClass):
    """A TvEpisode is a singular unit of television"""
    def __init__(self,information_repository,**kwargs):

        logging.debug("instantiated episode")
        self.episode_dict={}
        self.__dict__["episode_number"]=""
        self.__dict__["episode_title"]=""
        for key,value in kwargs.items():
            self.set_value(key,value)
        self.init_sources(information_repository)
        
    def set_value(self,key,value,scrub=False):
        try:
            if str(value).isdigit():
                value=int(value)
        except UnicodeEncodeError:
            pass
                       
        if key=="episode_number" and (isinstance(value,int) or len(value)>0):
            self.add_episode(value)
        elif scrub and (isinstance(value,str) or isinstance(value,unicode)):
            self.__dict__[key] = scrub_illegal(value)
        else:
            self.__dict__[key] = value

    def add_episode(self,episodeString):

        if type(episodeString)==type(int()):
            self.episode_dict[int(episodeString)]=""
        else:
            for episode_number in re.split(r"\D+",episodeString):
                self.episode_dict[int(episode_number)]=""
        
    def get_episodes():
        
        return self.episode_dict.keys()
        
    def get_titles(self,selected_number=None):

        if selected_number is not None:
            logging.debug("getting title for episode %s",selected_number)
        if not selected_number:
            #assuming all numbers
            for episode in self.episode_dict.keys():
                if not self.episode_dict.get(episode):             
                    if selected_number is None:
                        logging.debug("getting title for episode %s",episode)
                    self.__add_episode_title(episode)
        else:
            self.__add_episode_title(selected_number)
            
        return self.episode_dict.values()
    
#---Using Media Information
    
    def expand_template(self,template):

        template_dict = self.__dict__
        
        self["episode_number"] = ""
        self["episode_title"] = ""

        for episode_number,episode_title in self.episode_dict.iteritems():
            self.__append_piece("episode_number",
                            str(episode_number).zfill(cl_opt.episode_padding))
            
            self.__append_piece("episode_title",episode_title)
            
        season_pad = str(self["season_number"]).zfill(cl_opt.season_padding)
        
        if not self["episode_number"]:
            del self.__dict__["episode_number"]
            
        if not self["episode_title"]:
            del self.__dict__["episode_title"]
            
        scrubbed_title = scrub_illegal(self["episode_title"])
        scrubbed_showname = scrub_illegal(self["showname"])
        
        try:
            expanded_template = Template(template).substitute(self.__dict__,
                                                season_number=season_pad,
                                                episode_title=scrubbed_title,
                                                showname=scrubbed_showname)
        except KeyError:
            logging.warning("not all template pieces converted in %s",template)
            logging.warning(self)
            return None
        
        return expanded_template
    
    def exec_move(self,copy=False):
        if "oldpath" not in self.__dict__:
            self["oldpath"],self["abspath"] = self.set_move_command()
        
        
        
        if self["abspath"] is None:
                logging.error("empty destination for %s",self["oldpath"])                                
                self["abspath"] = self["oldpath"]
                return None
            
        if os.path.exists(self["abspath"]):
            logging.warning("about to overwrite %s, not executing move",
                            self["abspath"])
            return None
        
        target_dir = os.path.dirname(self["abspath"])
        
        if not cl_opt.pretend:
            if not os.path.exists(target_dir):
                try:
                    os.makedirs(target_dir)
                except Exception:
                    logging.error("error creating path: %s",target_dir)
                    return None
                
        logging.info("%s -to-> %s",self["oldpath"],self["abspath"])

        if not cl_opt.pretend:
            try:
                if not copy:
                    shutil.move(self["oldpath"],self["abspath"])
                else:
                    shutil.copy(self["oldpath"],self["abspath"])
            except Exception:
                logging.error("error moving file %s to %s",
                                self["oldpath"],
                                self["abspath"])

                self["abspath"] = self["oldpath"]

                
                
            
    def set_move_command(self,output_template=None):
        if output_template is None:
            output_template = cl_opt.output_template
            
        if cl_opt.write_to_folder:
            top_folder = cl_opt.output_dir
        
        else:
            top_folder = os.path.dirname(self["abspath"])
            
        new_template = (os.path.join(top_folder,output_template) + 
                                        os.path.splitext(self["abspath"])[1])
        
       
        old_path=self["abspath"]
        new_path=self.expand_template(new_template)
        
        return old_path,new_path
            
        
    
#---Utility
    
    def __add_episode_title(self,episode_number):

        foundTitle,proper_showname = self.repo.get_title(self["showname"],
                                                         self["season_number"],
                                                         episode_number)
                                                        
        self["showname"] = proper_showname
        
        self.episode_dict[episode_number] = foundTitle
        
        
                
    def __append_piece(self,media_key,piece):
        if piece is None:
            logging.debug("dropping None from %s",media_key)
            return None

        opt_sep = cl_opt.split_episode_char

        logging.debug("constructing %s by adding %s",
                        media_key,
                        unicode(piece))
                    
        current_value = self.__dict__[media_key]
        
        logging.debug("current value: %s",current_value)

        if len(current_value) > 0:
            logging.debug("adding seperator")
            self.__dict__[media_key] = current_value+opt_sep+unicode(piece)
                                                    
        else:
            self.__dict__[media_key] = unicode(piece)
        
        
        logging.debug("resulting value: %s",self.__dict__[media_key])        

        