#!/usr/local/bin/pythonw
# Rob Dennis - Lurchington Q. Delacroix
# Keasby - TV renamer
# Project Homepage: http://code.google.com/p/keasby/

#---Python Import
import os
import sys
import logging
#---GUI import
try:
    import wx
except ImportError:
    print "error importing wxPython. make sure it's downloaded and installed"
    print "eventually this will default back to a console batch mode"
    raise
#---Keasby Import
from core.keasby_options import cl_opt,cl_arg,keasby_options_instance
from core.media_list import MediaList
from core.media_interpreter import MediaInterpreter
from core.keasby_options import cl_opt,cl_arg
from core.keasby_source import KeasbyInfoRepo
from core.keasby_helper_func import keasby_multichoice

if cl_opt.debug or cl_opt.verbose:
    logging.basicConfig(level=logging.DEBUG,format='%(message)s')
elif cl_opt.show_output:
    logging.basicConfig(level=logging.INFO,format='%(message)s')
else:
    logging.basicConfig(level=logging.WARNING,format='%(message)s')


class myFrame(wx.Frame):
    def __init__(self,parent,id,title):
        wx.Frame.__init__(self,parent,id,title,size=(1000,300))
        
        top_panel = wx.Panel(self, wx.ID_ANY,size=self.GetSize())
        
        top_panel.SetBackgroundColour("GRAY")
        
        #---Top Level Directory Choice
        self.in_dir_ctrl = wx.DirPickerCtrl(top_panel,
                                            wx.ID_ANY,
                                            cl_opt.input_dir,
                                            style=wx.DIRP_DEFAULT_STYLE)
                                            
        self.in_dir_label = wx.StaticText(top_panel,
                                          wx.ID_ANY,
                                          "working directory")
                                        
        self.out_dir_ctrl = wx.DirPickerCtrl(top_panel,
                                            wx.ID_ANY,
                                            cl_opt.output_dir,
                                            style=wx.DIRP_DEFAULT_STYLE)
                                            
        self.out_dir_label = wx.StaticText(top_panel,
                                          wx.ID_ANY,
                                          "output directory")

        
        #---Rename Template
        self.rename_ctrl = wx.TextCtrl(top_panel,wx.ID_ANY,
                                       cl_opt.output_template)
        self.rename_label = wx.StaticText(top_panel,wx.ID_ANY,"rename string")

        #---Manual Specification
        self.showname_chk = wx.CheckBox(top_panel,wx.ID_ANY,"use?")
        self.showname_chk.SetValue(bool(cl_opt.manual_showname))
        self.showname_chk.SetToolTip(wx.ToolTip("Check to use manual showname"))
        
        self.season_chk = wx.CheckBox(top_panel,wx.ID_ANY,"use?")
        self.season_chk.SetValue(bool(cl_opt.manual_season))
        self.season_chk.SetToolTip(wx.ToolTip("Check to use manual season"))
        
        self.showname_ctrl = wx.TextCtrl(top_panel,
                                         wx.ID_ANY,str(cl_opt.manual_showname))
        self.showname_label = wx.StaticText(top_panel,wx.ID_ANY,
                                            "manual showname")
        self.showname_ctrl.SetEditable(self.showname_chk.IsChecked())
        
        self.season_ctrl = wx.TextCtrl(top_panel,wx.ID_ANY,
                                       str(cl_opt.manual_season))
        self.season_label = wx.StaticText(top_panel,wx.ID_ANY,
                                          "manual season number")
        self.season_ctrl.SetEditable(self.season_chk.IsChecked())

        #---Miscellaneous Checks
        self.recursive_chk = wx.CheckBox(top_panel,wx.ID_ANY,"recursive?")
        self.recursive_chk.SetValue(cl_opt.recursive_rename)
        self.recursive_chk.SetToolTip(wx.ToolTip("When checked, will operate on all subfolders"))
        
        self.unattended_chk = wx.CheckBox(top_panel,wx.ID_ANY,"unattended mode?")
        self.unattended_chk.SetValue(cl_opt.unattended)
        self.unattended_chk.SetToolTip(wx.ToolTip("Leave this unchecked to prompt for input on change and to continue on"))
        
        self.move_folder_chk = wx.CheckBox(top_panel,wx.ID_ANY,"move files to output folder?")
        self.move_folder_chk.SetValue(cl_opt.write_to_folder)
        self.move_folder_chk.SetToolTip(wx.ToolTip("Leave this unchecked to rename files within current folder, if your template has a directory specified, it will make subfolders"))
        
        self.pretend_chk = wx.CheckBox(top_panel,wx.ID_ANY,"pretend mode?")
        self.pretend_chk.SetValue(cl_opt.pretend)
        self.pretend_chk.SetToolTip(wx.ToolTip("Leave this checked to test results with no changes made"))

        #---Buttons
        self.keasby_emcee = KeasbyEmcee(self)
        self.start_btn = wx.Button(top_panel,wx.ID_ANY,"Start")
        self.start_btn.Bind(wx.EVT_BUTTON,self.OnStartBtnHit)
        self.start_btn.SetToolTip(wx.ToolTip("Start. This. Process."))
        
        self.echo_btn = wx.Button(top_panel,wx.ID_ANY,"Echo")
        self.echo_btn.Bind(wx.EVT_BUTTON,self.OnEchoBtnHit)
        self.echo_btn.SetToolTip(wx.ToolTip("Echo. These. cl_options."))
        
        #---Box Sizers
        total_box = wx.BoxSizer(wx.VERTICAL)
        
        total_box.Add(top_panel,1,wx.EXPAND)
        
        self.SetAutoLayout(True)
        self.SetSizer(total_box)
        
        top_box = wx.BoxSizer(wx.VERTICAL)    
        
        inputPickerBox = wx.BoxSizer(wx.HORIZONTAL)
        outputPickerBox = wx.BoxSizer(wx.HORIZONTAL)
        renameBox = wx.BoxSizer(wx.HORIZONTAL)
        manualBox = wx.BoxSizer(wx.HORIZONTAL)
        renameFolderBox = wx.BoxSizer(wx.HORIZONTAL)
        miscBox = wx.BoxSizer(wx.HORIZONTAL)
        
        #---Individual Box Sizers
        b = 8
        
        inputPickerBox.Add(self.in_dir_label,0,wx.EXPAND|wx.ALL,b)
        inputPickerBox.Add(self.in_dir_ctrl,1,wx.EXPAND|wx.ALL,b)
        inputPickerBox.Add(self.recursive_chk,0,
                           wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL,b)

        outputPickerBox.Add(self.out_dir_label,0,wx.EXPAND|wx.ALL,b)
        outputPickerBox.Add(self.out_dir_ctrl,1,wx.EXPAND|wx.ALL,b)
        outputPickerBox.Add(self.move_folder_chk,0,
                            wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL|wx.ALL,b)
        
        renameBox.Add(self.rename_label,0,wx.EXPAND|wx.ALL,b)
        renameBox.Add(self.rename_ctrl,1,wx.EXPAND|wx.ALL,b)
        
        manualBox.Add(self.showname_label,0,wx.EXPAND|wx.ALL,b)
        manualBox.Add(self.showname_ctrl,1,wx.EXPAND|wx.ALL,b)
        manualBox.Add(self.showname_chk,1,wx.
                      ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL,b)
                    
        manualBox.Add(self.season_label,0,wx.EXPAND|wx.ALL,b)
        manualBox.Add(self.season_ctrl,1,wx.EXPAND|wx.ALL,b)
        manualBox.Add(self.season_chk,1,
                      wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL,b)
        
        miscBox.Add(self.unattended_chk,1,
                    wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL,b)
        miscBox.Add(self.pretend_chk,1,
                    wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL,b)
        miscBox.Add(self.start_btn,1,wx.ALIGN_CENTER_VERTICAL,b)
        miscBox.Add(self.echo_btn,1,wx.ALIGN_CENTER_VERTICAL,b)
        
        top_box.Add(inputPickerBox,0,wx.EXPAND|wx.ALL,b)
        top_box.Add(outputPickerBox,0,wx.EXPAND|wx.ALL,b)
        top_box.Add(renameBox,0,wx.EXPAND|wx.ALL,b)
        top_box.Add(manualBox,0,wx.EXPAND|wx.ALL,b)
        top_box.Add(miscBox,0,wx.EXPAND|wx.ALL,b)
        
        top_panel.SetAutoLayout(True)
        top_panel.SetSizer(top_box)
        
        self.Layout()
        
        #---Events
        wx.EVT_DIRPICKER_CHANGED(self,self.in_dir_ctrl.GetId(),
                                 self.OnInputDirChanged)
        wx.EVT_DIRPICKER_CHANGED(self,self.out_dir_ctrl.GetId(),
                                 self.OnOutputDirChanged)
        wx.EVT_CHECKBOX(self,self.recursive_chk.GetId(),self.OnRecursiveCheck)
        wx.EVT_CHECKBOX(self,self.unattended_chk.GetId(),
                        self.OnUnattendedCheck)
        wx.EVT_CHECKBOX(self,self.pretend_chk.GetId(),self.OnPretendCheck)        
        wx.EVT_CHECKBOX(self,self.move_folder_chk.GetId(),self.OnFolderCheck)
        wx.EVT_CHECKBOX(self,self.showname_chk.GetId(),
                        self.OnManualShownameCheck)
        wx.EVT_CHECKBOX(self,self.season_chk.GetId(),self.OnManualSeasonCheck)
        wx.EVT_TEXT(self,self.season_ctrl.GetId(),self.OnSeasonText)
        wx.EVT_TEXT(self,self.showname_ctrl.GetId(),self.OnShownameText)
        wx.EVT_TEXT(self,self.rename_ctrl.GetId(),self.OnRenameText)


    
    def OnCloseWindow(self, event):
        self.destroy()
        
    def OnInputDirChanged(self,event):
        logging.info("changed input dir to %s"%event.GetPath())
        cl_opt.input_dir = event.GetPath()
        
    def OnOutputDirChanged(self,event):
        logging.info("changed output dir to %s"%event.GetPath())
        cl_opt.output_dir = event.GetPath()
        
        
    def OnStartBtnHit(self,event):
        logging.info("begun command for directory: %s",cl_opt.input_dir)
        self.keasby_emcee.run()
        logging.info("completed command")
    
    def OnEchoBtnHit(self,event):
        print keasby_options_instance.echo(cl_opt)
        
    def OnRecursiveCheck(self,event):
        cl_opt.recursive_rename = event.IsChecked()
        
    def OnUnattendedCheck(self,event):
        cl_opt.unattended = event.IsChecked()

    def OnPretendCheck(self,event):
        cl_opt.pretend = event.IsChecked()
        
    def OnFolderCheck(self,event):
        cl_opt.write_to_folder = event.IsChecked()
        
    def OnManualShownameCheck(self,event):
        if event.IsChecked():
            cl_opt.manual_showname = self.showname_ctrl.GetValue()
            self.showname_ctrl.SetEditable(True)
            
        else:
            cl_opt.manual_showname = None
            self.showname_ctrl.SetEditable(False)
            
                    
    def OnManualSeasonCheck(self,event):
        if event.IsChecked():
            cl_opt.manual_season = self.season_ctrl.GetValue()
            self.season_ctrl.SetEditable(True)

        else:
            cl_opt.manual_season = None
            self.season_ctrl.SetEditable(False)
            
    def OnShownameText(self,event):
        cl_opt.manual_showname = self.showname_ctrl.GetValue()
        
    def OnSeasonText(self,event):
        cl_opt.manual_season = self.season_ctrl.GetValue()
        
    def OnRenameText(self,event):
        cl_opt.output_template = self.rename_ctrl.GetValue()
        

class KeasbyEmcee():
    
    def __init__(self,parent):
        self.parent = parent
        self.repo = KeasbyInfoRepo()
        
    def run(self):
        logging.debug("in KeasbyEmcee.run()")
        logging.debug("top_level_dir = %s",cl_opt.input_dir)
        
        for root,dirs,files in os.walk(cl_opt.input_dir):
            file_list = MediaList(root).get_files()
            logging.info("checking %s for files",root)
            if len(file_list)>0:

                logging.debug("----")
                logging.debug("found %s files",len(file_list))
                logging.debug("----")

                media_list = []
                
                for file in file_list:
                    media_int = MediaInterpreter(file,self.repo,self.parent)
                    try:
                        media_list.append(media_int.search_title())
                    except TypeError:
                        logging.info("no usable file found, skipping")
                    
                self.__confirm_list(media_list)

            if not cl_opt.recursive_rename:
                break
                
    def __confirm_list(self,media_list):

        media_whitelist = []
        printed_list = []
        
        
        
        for media_file in media_list:

            old_path,new_path = media_file.set_move_command()

            if new_path:
                base_dir,base_file = os.path.split(old_path)
                media_whitelist.append(media_file)
                printed_list.append("%s -> %s"%(base_file,new_path))
                                                                
     
        if media_whitelist and printed_list:
            selected_media = keasby_multichoice(printed_list,
                                                window_caption=base_dir,
                                                force=cl_opt.unattended)
        else:
            logging.info("no usable files")
            return None
        
        if not selected_media:
            return None
        
        for selection in selected_media:
            logging.info("executing move command:")
            logging.info(media_whitelist[selection])
            media_list[selection].exec_move()
                                            
                                            




class myApp(wx.App):
    def OnInit(self):
        frame = myFrame(None,wx.ID_ANY,"Keasby")
        frame.Show(True)
        
        return True
        
        
def main():
    app = myApp(0)
    app.MainLoop()
    
main()